MODULE SVGAInfo;
(*-------------------------------------------------------------------------*)
(* Author: Alan P. Burke, Oghma Systems Ltd., RR#1, Chemin de la Mine,     *)
(*         Chelsea, Quebec, Canada, J0X 1N0.   BIX userid : aburke.        *)
(*                                                                         *)
(* This program uses functions 4F00H and 4F01H of interrupt 10H, as        *)
(* documented in the VESA SVGA standard #VS900602, to display information  *)
(* about modes supported. You may use or modify this program freely        *)
(* provided that you credit the author and clearly mark any modifications  *)
(* which you make. The author makes no warranty concerning the correctness *)
(* or usefulness of the program and shall not be held responsible in any   *)
(* way for damages which might be caused through its use, whether they be  *)
(* direct or indirect.                                                     *)
(* The program is written for the Stony Brook Modula-2 compiler,           *)
(* version 2.2. The procedure "MemZero" in module "Memory" has not been    *)
(* included; it's sole purpose is to clear a block of memory.              *)
(*-------------------------------------------------------------------------*)
FROM SYSTEM IMPORT REGISTERS, INT, BYTE, WORD, ADR, ADDRESS;
FROM InOut IMPORT Write, WriteString, WriteCard, WriteHex, WriteLn;
FROM Memory IMPORT MemZero;
(*-------------------------------------------------------------------------*)
TYPE
  StrPtr = POINTER TO ARRAY [0..255] OF CHAR;
  ModePtr = POINTER TO ARRAY [0..511] OF CARDINAL;
  ByteSet = SET OF [0..7];
(*-------------------------------------------------------------------------*)
VAR
  modeIndex : CARDINAL;
  regs : REGISTERS;
  buffAddr : ADDRESS;
  buffer : RECORD
             signature    : ARRAY [0..3] OF CHAR;  (* 'VESA' *)
             version      : CARDINAL;
             oemNamePtr   : StrPtr;
             capabilities : ARRAY [0..1] OF BITSET;
             modesPtr     : ModePtr;
             totalMemory  : CARDINAL;
             reserved     : ARRAY [1..242] OF BYTE;
           END;
(*-------------------------------------------------------------------------*)
PROCEDURE GetModeInfo (mode : CARDINAL);
VAR
  i, colors : CARDINAL;
  regs : REGISTERS;
  modeInfoBlock :
    RECORD
      modeAttributes     : BITSET;
      winAAttributes     : ByteSet;
      winBAttributes     : ByteSet;
      winGranularity     : CARDINAL;
      winSize            : CARDINAL;
      winASegment        : CARDINAL;
      winBSegment        : CARDINAL;
      winFuncPtr         : ADDRESS;
      bytesPerScanLine   : CARDINAL;
      xResolution        : CARDINAL;
      yResolution        : CARDINAL;
      xCharSize          : BYTE;
      yCharSize          : BYTE;
      numberOfPlanes     : BYTE;
      bitsPerPixel       : BYTE;
      numberOfBanks      : BYTE;
      memoryModel        : BYTE;
      bankSize           : BYTE;
      numberOfImagePages : BYTE;
      filler             : ARRAY [0..225] OF BYTE;
    END;
BEGIN (* GetModeInfo *)
  WITH regs DO
    AX := 4F01H;
    CX := mode;
    buffAddr := ADR(modeInfoBlock);
    ES := buffAddr.SEGMENT;
    DI := buffAddr.OFFSET;
    MemZero(modeInfoBlock);
    INT(10H,regs);
    IF ORD(AL) = 4FH THEN
      IF ORD(AH) = 00H THEN
        WITH modeInfoBlock DO
          IF (3 IN modeAttributes) THEN
            WriteString(' Color ');
          ELSE
            WriteString(' Mono  ');
          END (* if *);
          IF (4 IN modeAttributes) THEN
            WriteString('Graphics ');
          ELSE
            WriteString('Text     ');
          END (* if *);
          IF (1 IN modeAttributes) THEN
            WriteCard(xResolution,5);
            WriteCard(yResolution,5);
            colors := 1;
            FOR i := 1 TO ORD(bitsPerPixel) DO
              colors := colors*2;
            END (* for *);
            WriteCard(colors,4);
            WriteString(' chars: ');
            WriteCard(ORD(xCharSize),1);
            WriteString('x');
            WriteCard(ORD(yCharSize),1);
            WriteLn;
          END (* if *);
          IF (0 IN winAAttributes) THEN
            WriteString('      Window A is');
            IF (1 IN winAAttributes) THEN
              WriteString(' readable');
            END (* if *);
            IF (2 IN winAAttributes) THEN
              WriteString(' writeable');
            END (* if *);
            WriteString(' at segment:');
            WriteHex(winASegment,4);
            WriteLn;
          END (* if *);
          IF (0 IN winBAttributes) THEN
            WriteString('      Window B is');
            IF (1 IN winBAttributes) THEN
              WriteString(' readable');
            END (* if *);
            IF (2 IN winBAttributes) THEN
              WriteString(' writeable');
            END (* if *);
            WriteString(' at segment:');
            WriteHex(winBSegment,4);
            WriteLn;
          END (* if *);
          WriteString('      ');
          WriteString('Granularity:');
          WriteCard(winGranularity,1);
          WriteString('KB, Size:');
          WriteCard(winSize,1);
          WriteString('KB, Bytes/scanline:');
          WriteCard(bytesPerScanLine,1);
          WriteLn;
          IF (1 IN modeAttributes) THEN
            WriteString('      ');
            WriteString('Planes:');
            WriteCard(ORD(numberOfPlanes),1);
            WriteString(', Banks:');
            WriteCard(ORD(numberOfBanks),1);
            WriteString(', BankSize:');
            WriteCard(ORD(bankSize),1);
            WriteString(', Images:');
            WriteCard(ORD(numberOfImagePages),1);
            WriteString(', Mem Model:');
            CASE ORD(memoryModel) OF
              00H       : WriteString(' Text');
            | 01H       : WriteString(' CGA graphics');
            | 02H       : WriteString(' Hercules graphics');
            | 03H       : WriteString(' 4-plane planar');
            | 04H       : WriteString(' Packed pixel');
            | 05H       : WriteString(' Non-chain 4, 256 color');
            | 06H..00FH : WriteString(' VESA reserved');
            | 10H..0FFH : WriteString(' OEM reserved');
            ELSE          WriteString(' Unknown');
            END (* case *);
          END (* if *);
          WriteLn;
          IF NOT (0 IN modeAttributes) THEN
            WriteString('      Not supported by monitor');
            WriteLn;
          END (* if *);
          IF NOT (1 IN modeAttributes) THEN
            WriteString('      No extended information');
            WriteLn;
          END (* if *);
          IF NOT (2 IN modeAttributes) THEN
            WriteString('      No BIOS output functions');
            WriteLn;
          END (* if *);
        END (* with *);
      ELSIF ORD(AH) = 01H THEN
        WriteString('GetModeInfo Function failed'); WriteLn;
      ELSE
        WriteString('GetModeInfo Undocumented return code'); WriteLn;
      END (* if *);
    ELSE
      WriteString('GetModeInfo Function not supported'); WriteLn;
    END (* if *);
  END (* with *);
END GetModeInfo;
(*-------------------------------------------------------------------------*)
BEGIN (* SVGAInfo *)
  WriteString('SVGAInfo Utility V1.00 from Alan Burke, Oghma Systems Ltd.');
  WriteLn;
  WITH regs DO
    AX := 4F00H;
    buffAddr := ADR(buffer);
    ES := buffAddr.SEGMENT;
    DI := buffAddr.OFFSET;
    INT(10H,regs);
    IF ORD(AL) = 4FH THEN
      IF ORD(AH) = 00H THEN
        WITH buffer DO
          WriteString(signature);
          WriteString(', version: ');
          WriteHex(version,4);
          WriteString(', OEM name: "');
          WriteString(oemNamePtr^);
          WriteString('", 64KB memory blks: ');
          WriteCard(totalMemory,1);
          WriteLn;
          modeIndex := 0;
          WHILE modesPtr^[modeIndex] # 0FFFFH DO
            WriteHex(modesPtr^[modeIndex],5);
            GetModeInfo(modesPtr^[modeIndex]);
            INC(modeIndex);
          END (* while *);
        END (* with *);
      ELSIF ORD(AH) = 01H THEN
        WriteString('GetSVGAInfo Function failed'); WriteLn;
      ELSE
        WriteString('GetSVGAInfo Undocumented return code'); WriteLn;
      END (* if *);
    ELSE
      WriteString('GetSVGAInfo Function not supported'); WriteLn;
      WriteString('Verify that the SVGA driver (if any) has been installed');
      WriteLn;
    END (* if *);
  END (* with *);
END SVGAInfo.
